# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/job.ipynb.

# %% auto 0
__all__ = ['get_jobs', 'add_job', 'generate_body_remote_domostats', 'generate_body_watchdog_generic', 'update_job',
           'update_job_trigger']

# %% ../../nbs/routes/job.ipynb 2
from typing import Union
import httpx
from pprint import pprint

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda


# %% ../../nbs/routes/job.ipynb 3
# get RemoteDomostats job names
async def get_jobs(auth: dmda.DomoFullAuth,
                   application_id: str,
                   debug_api: bool = False,
                   session: Union[httpx.AsyncClient, httpx.AsyncClient, None] = None)-> rgd.ResponseGetData:
    
    offset_params = {
            'offset': 'offset',
            'limit': 'limit'}

    url = f'https://{auth.domo_instance}.domo.com/api/executor/v2/applications/{application_id}/jobs'

    if debug_api:
        print(url)

    def arr_fn(res) -> list[dict]:
        return res.response.get('jobs')


    # deprecated from looper 
    # def alter_maximum_fn(res):
    #     return res.response.get('totalResults')

    return await gd.looper(auth=auth,
                           method='GET',
                           url=url,
                           arr_fn=arr_fn,
                           loop_until_end=True,
                           offset_params=offset_params,
                           session=session,
                           debug_api=debug_api)


# create the new RemoteDomostats job
async def add_job(auth: dmda.DomoFullAuth,
                  body: dict,
                  application_id: str,
                  session: Union[httpx.AsyncClient, httpx.AsyncClient, None] = None,
                  debug_api: bool = False
                  )-> rgd.ResponseGetData:

    url = f'https://{auth.domo_instance}.domo.com/api/executor/v1/applications/{application_id}/jobs'

    if debug_api:
        print(url)

    return await gd.get_data(
        auth=auth,
        url=url,
        method='POST',
        body=body,
        debug_api=debug_api,
        session=session
    )


def generate_body_remote_domostats(target_instance: str,
                                   report_dict: dict,
                                   output_dataset_id: str,
                                   account_id: str,
                                   schedule_ls: list,
                                   execution_timeout: int = 1440,
                                   debug_api: bool = False):

    instance_url = f"{target_instance}.domo.com"

    body = {
        "jobName": instance_url,
        "jobDescription": f'Get Remote stat from {instance_url}',
        "executionTimeout": execution_timeout,
        "executionPayload": {
            "remoteInstance": instance_url,
            "policies": report_dict,
            "metricsDatasetId": output_dataset_id},
        "accounts": [account_id],
        "executionClass": "com.domo.executor.subscriberstats.SubscriberStatsExecutor",
        "resources": {
            "requests": {"memory": "256M"},
            "limits": {"memory": "256M"}},
        "triggers": schedule_ls
    }

    if debug_api:
        pprint(body)

    return body


def generate_body_watchdog_generic(job_name: str,
                                        notify_user_ids_ls: list,
                                        notify_group_ids_ls: list,
                                        notify_emails_ls: list,
                                        entity_ids_ls: list,
                                        entity_type : str,
                                        metric_dataset_id: str,
                                        schedule_ls: list,
                                        job_type : str,
                                        execution_timeout: int = 1440,
                                        debug_api: bool = False):

    body = {
        "jobName": job_name,
        "jobDescription": f'Watchdog for {job_name}',
        "executionTimeout": execution_timeout,
        "accounts": [],
        "executionPayload": {
            "notifyUserIds": notify_user_ids_ls or [],
            "notifyGroupIds": notify_group_ids_ls or [],
            "notifyEmailAddresses": notify_emails_ls or [],
        "watcherParameters": {
          "entityIds": entity_ids_ls,
          "type": job_type,
          "entityType": entity_type
            },
        "metricsDatasetId": metric_dataset_id
          },
        "resources": {
            "requests": {"memory": "256Mi"},
            "limits": {"memory": "256Mi"}
        },
        "triggers": schedule_ls
    }

    if debug_api:
        pprint(body)

    return body

# update the job
async def update_job(auth: dmda.DomoFullAuth,
                  body: dict,
                  job_id :str,
                  application_id: str,
                  session: Union[httpx.AsyncClient, httpx.AsyncClient, None] = None,
                  debug_api: bool = False
                  )-> rgd.ResponseGetData:

    url = f'https://{auth.domo_instance}.domo.com/api/executor/v1/applications/{application_id}/jobs/{job_id}'

    if debug_api:
        print(url)

    return await gd.get_data(
        auth=auth,
        url=url,
        method='PUT',
        body=body,
        debug_api=debug_api,
        session=session
    )

#update trigger

async def update_job_trigger(auth: dmda.DomoFullAuth,
                  body: dict,
                  job_id :str,
                  trigger_id : str,  
                  application_id: str,
                  session: Union[httpx.AsyncClient, httpx.AsyncClient, None] = None,
                  debug_api: bool = False
                  )-> rgd.ResponseGetData:

    url = f'https://{auth.domo_instance}.domo.com/api/executor/v1/applications/{application_id}/jobs/{job_id}/triggers/{trigger_id}'

    if debug_api:
        print(url)

    return await gd.get_data(
        auth=auth,
        url=url,
        method='PUT',
        body=body,
        debug_api=debug_api,
        session=session
    )



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/account.ipynb.

# %% auto 0
__all__ = ['get_accounts', 'GetAccount_NoMatch', 'get_account_from_id', 'AccountConfig_InvalidDataProvider', 'get_account_config',
           'UpdateAccount_Error', 'update_account_config', 'update_account_name', 'create_account', 'delete_account',
           'ShareAccount', 'ShareAccount_V1_AccessLevel', 'ShareAccount_V2_AccessLevel',
           'generate_share_account_payload_v1', 'generate_share_account_payload_v2', 'share_account_v2',
           'share_account_v1', 'get_share_account_v2']

# %% ../../nbs/routes/account.ipynb 3
from typing import Union
from enum import Enum
import httpx
import asyncio

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as de

# %% ../../nbs/routes/account.ipynb 4
async def get_accounts(auth: dmda.DomoAuth,
                       debug_api: bool = False, 
                       session: Union[httpx.AsyncClient, httpx.AsyncClient, None] = None) -> rgd.ResponseGetData:
    """retrieve a list of all the accounts the user has read access to.  Note users with "Manage all accounts" will retrieve all account objects"""
    
    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/accounts"

    return await gd.get_data(
        auth=auth,
        url=url,
        method='GET',
        debug_api=debug_api,
        session=session
    )

# %% ../../nbs/routes/account.ipynb 7
class GetAccount_NoMatch(de.DomoError):
    def __init__(self, account_id, domo_instance, status= None, function_name = 'get_account_from_id'):

        message = f"account_id {account_id} not found"
        
        super().__init__(message = message, status = status, function_name = function_name , domo_instance = domo_instance)
    
async def get_account_from_id(auth: dmda.DomoAuth, account_id: int,
                              debug_api: bool = False, session: httpx.AsyncClient = None) -> rgd.ResponseGetData:
    """retrieves metadata about an account"""

    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/accounts/{account_id}?unmask=true"

    if debug_api:
        print(url)

    res = await gd.get_data(
        auth=auth,
        url=url,
        method='GET',
        debug_api=debug_api,
        session=session, 
        timeout = 20 # occasionally this API has a long response time
    )

    if not res.is_success and (res.response == 'Forbidden' or res.response == 'Not Found'):
        raise GetAccount_NoMatch(
            account_id=account_id, domo_instance=auth.domo_instance, status=res.status)
    
    return res




# %% ../../nbs/routes/account.ipynb 11
class AccountConfig_InvalidDataProvider(de.DomoError):
    def __init__(self, account_id:str, data_provider_type:str, status, domo_instance: str, function_name = 'route.get_account_config'):
        message = f"account {account_id} with data_provider_type '{data_provider_type}' does not return a config"

        super().__init__( message = message, status = status, function_name = function_name, domo_instance = domo_instance)

async def get_account_config(auth: dmda.DomoAuth,
                             account_id: int,
                             data_provider_type: str ,
                             debug_api: bool = False, 
                             session: Union[httpx.AsyncClient, httpx.AsyncClient, None] = None) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/providers/{data_provider_type}/account/{account_id}?unmask=true"

    if debug_api:
        print(url)

    res = await gd.get_data(
        auth=auth,
        url=url,
        method='GET',
        debug_api=debug_api,
        session=session
    )

    if res.response == {} and res.is_success:
        raise AccountConfig_InvalidDataProvider(account_id= account_id, data_provider_type= data_provider_type, domo_instance=auth.domo_instance, status = res.status)
    
    return res

# %% ../../nbs/routes/account.ipynb 15
class UpdateAccount_Error(de.DomoError):
    def __init__(self, status, response, account_id,
                 domo_instance,
                 info=None,
                 ):

        message = f"unable to update account {account_id} - {response} { (' - ' + info) or ''}"

        super().__init__(status=status, message=message, domo_instance=domo_instance)


async def update_account_config(auth: dmda.DomoAuth,
                                account_id: int,
                                config_body: dict,
                                data_provider_type: str,
                                debug_api: bool = False,
                                session: httpx.AsyncClient = None) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/providers/{data_provider_type}/account/{account_id}"

    if debug_api:
        print(url)

    res = await gd.get_data(
        auth=auth,
        url=url,
        method='PUT',
        body=config_body,
        debug_api=debug_api,
        session=session
    )

    if res.status == 400 and res.response == 'Bad Request':
        raise UpdateAccount_Error(status=res.status, response=res.response,
                                  account_id=account_id,
                                  info='updating config | use debug_api to check the URL - ', domo_instance=auth.domo_instance)

        res.is_success = False

    if res.status != 200:
        raise UpdateAccount_Error(status=res.status, response=res.response,
                                  account_id=account_id,
                                  info='updating account config', domo_instance=auth.domo_instance)

        res.is_success = False

    return res


# %% ../../nbs/routes/account.ipynb 18
async def update_account_name(auth: dmda.DomoAuth,
                              account_id: int,
                              account_name: str,
                              debug_api: bool = False, 
                              session: httpx.AsyncClient = None) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/accounts/{account_id}/name"
    
    if debug_api:
        print(url)

    return await gd.get_data(
        auth=auth,
        url=url,
        method='PUT',
        body=account_name,
        content_type = "text/plain",
        debug_api=debug_api,
        session=session
    )

# %% ../../nbs/routes/account.ipynb 19
async def create_account(auth:dmda.DomoAuth, config_body:dict,
                         debug_api: bool = False, session: httpx.AsyncClient = None) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/accounts"

    if debug_api:
        print(url)

    attempt = 1
    res = None

    while attempt <= 3:
        res = await gd.get_data(
            auth=auth,
            url=url,
            method='POST',
            body = config_body,
            debug_api=debug_api,
            session=session
        )

        if res.is_success:
            return res
        
        attempt += 1
        await asyncio.sleep(3)
    
    return res

# %% ../../nbs/routes/account.ipynb 20
async def delete_account(auth:dmda.DomoAuth,
                         account_id: str,
                         debug_api: bool = False, 
                         session: httpx.AsyncClient = None) -> rgd.ResponseGetData:
    
    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/accounts/{account_id}"

    if debug_api:
        print(url)

    return await gd.get_data(
        auth=auth,
        url=url,
        method='DELETE',
        debug_api=debug_api,
        session=session
    )

# %% ../../nbs/routes/account.ipynb 22
class ShareAccount():
    pass


class ShareAccount_V1_AccessLevel(ShareAccount, Enum):
    CAN_VIEW = 'READ'


class ShareAccount_V2_AccessLevel(ShareAccount, Enum):
    CAN_VIEW = 'CAN_VIEW'
    CAN_EDIT = 'CAN_EDIT'
    CAN_SHARE = 'CAN_SHARE'


def generate_share_account_payload_v1(
    access_level: ShareAccount,
    user_id: int = None,
    group_id: int = None
):
    if user_id:
        return {"type": "USER", "id": int(user_id), "permissions": [access_level.value]}
    if group_id:
        return {"type": "GROUP", "id": int(group_id), "permissions": [access_level.value]}


def generate_share_account_payload_v2(
    access_level: ShareAccount,
    user_id: int = None,
    group_id: int = None
):

    if user_id:
        return {"type": "USER", "id": int(user_id), "accessLevel": access_level.value}

    if group_id:
        return {"type": "GROUP", "id": int(group_id), "accessLevel": access_level.value}


# %% ../../nbs/routes/account.ipynb 24
async def share_account_v2(auth: dmda.DomoAuth,
                           account_id: str,
                           share_payload: dict,
                           debug_api: bool = False,
                           session: httpx.AsyncClient = None
                           ):

    url = f"https://{auth.domo_instance}.domo.com/api/data/v2/accounts/share/{account_id}"

    return await gd.get_data(
        auth=auth,
        url=url,
        method='PUT',
        body=share_payload,
        debug_api=debug_api,
        session=session
    )

#v1 may have been deprecated.  used to be tied to group beta
async def share_account_v1(auth: dmda.DomoAuth,
                           account_id: str,
                           share_payload: dict,
                           debug_api: bool = False,
                           session: httpx.AsyncClient = None
                           ):

    url = f"https://{auth.domo_instance}.domo.com/api/data/v1/accounts/{account_id}/share"

    return await gd.get_data(
        auth=auth,
        url=url,
        method='PUT',
        body=share_payload,
        debug_api=debug_api,
        session=session
    )




# %% ../../nbs/routes/account.ipynb 25
async def get_share_account_v2(auth: dmda.DomoAuth,
                           account_id: str,
                           debug_api: bool = False,
                           session: httpx.AsyncClient = None
                           ):

    url = f"https://{auth.domo_instance}.domo.com/api/data/v2/accounts/share/{account_id}"

    return await gd.get_data(
        auth=auth,
        url=url,
        method='GET',
        debug_api=debug_api,
        session=session
    )




# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/routes/card.ipynb.

# %% auto 0
__all__ = ['CardSearch_NotFoundError', 'get_kpi_definition', 'get_card_metadata', 'generate_body_search_cards_admin_summary',
           'search_cards_admin_summary']

# %% ../../nbs/routes/card.ipynb 2
import httpx

import domolibrary.client.get_data as gd
import domolibrary.client.ResponseGetData as rgd
import domolibrary.client.DomoAuth as dmda
import domolibrary.client.DomoError as de


# %% ../../nbs/routes/card.ipynb 3
class CardSearch_NotFoundError(de.DomoError):
    def __init__(self, card_id,
                 domo_instance, 
                 function_name,status,
                  message = None,
                 ):
        super().__init__(status=status,
                         message=f"card {card_id} not found", domo_instance=domo_instance, function_name=function_name)


# %% ../../nbs/routes/card.ipynb 4
async def get_kpi_definition(auth: dmda.DomoAuth, card_id: str, debug_api: bool = False, session: httpx.AsyncClient = None) -> rgd.ResponseGetData:

    url = f"https://{auth.domo_instance}.domo.com/api/content/v3/cards/kpi/definition"

    body = {"urn": card_id}

    res = await gd.get_data(
        auth=auth,
        url=url,
        method='PUT',
        body=body,
        debug_api=debug_api,
        session = session
    )

    if not res.is_success and res.response == 'Not Found':
        raise CardSearch_NotFoundError(card_id=card_id,
                                       status=res.status,
                                       domo_instance=auth.domo_instance, 
                                       function_name='get_kpi_definition')

    return res


# %% ../../nbs/routes/card.ipynb 7
async def get_card_metadata(auth: dmda.DomoAuth, card_id: str,
                            debug_api: bool = False, 
                            session: httpx.AsyncClient = None) -> rgd.ResponseGetData:
    optional_params = "metadata,certification,datasources,owners,problems"
    
    url = f"https://{auth.domo_instance}.domo.com/api/content/v1/cards?urns={card_id}&parts={optional_params}"

    res = await gd.get_data(
        auth=auth,
        url=url,
        method='GET',
        debug_api=debug_api,
        session=session
    )

    if res.is_success and len(res.response) == 0:
        raise CardSearch_NotFoundError(card_id=card_id,
                                       status=res.status,
                                       domo_instance=auth.domo_instance,
                                       function_name='get_kpi_definition')

    res.response = res.response[0]

    return res


# %% ../../nbs/routes/card.ipynb 10
def generate_body_search_cards_admin_summary(page_ids: [str] = None,
                                             #  searchPages: bool = True,
                                             card_search_text: str = None,
                                             page_search_text: str = None) -> dict:
    body = {
        "ascending": True,
        "orderBy": "cardTitle"
    }

    if card_search_text:
        body.update({'cardTitleSearchText': card_search_text,
                    "includeCardTitleClause": True})

    if page_search_text:
        body.update({'pageTitleSearchText': page_search_text,
                    "includePageTitleClause": True, "notOnPage": False})

    if page_ids:
        body.update({'pageIds': page_ids})

    return body


# %% ../../nbs/routes/card.ipynb 11
async def search_cards_admin_summary(auth: dmda.DomoAuth,
                                     body: dict,
                                     maximum :int = None,
                                     debug_api: bool = False,
                                     debug_loop: bool = False,
                                     session : httpx.AsyncClient  = None,
                                     wait_sleep: int = 3

                                     ) -> rgd.ResponseGetData:

    limit = 100
    offset = 0
    loop_until_end = False if maximum else True

    url = f'https://{auth.domo_instance}.domo.com/api/content/v2/cards/adminsummary?skip={offset}&limit={limit}'
    
    offset_params = {
        "offset": "skip",
        "limit": "limit",
    }

    def arr_fn(res):
        return res.response.get('cardAdminSummaries')

    res = await gd.looper(
        auth=auth,
        method="POST",
        url=url,
        arr_fn=arr_fn,
        offset_params=offset_params,
        limit=limit,
        skip=offset,
        body = body,
        maximum = maximum,
        session=session,
        debug_api=debug_api,
        debug_loop=debug_loop,
        loop_until_end=loop_until_end,
        wait_sleep=wait_sleep
    )
    return res

